#include <string>
#include <vector>
#include <iostream>

namespace PalmDateBook {

class TFieldType {
public:
	unsigned int m_fieldType;
	TFieldType() {
		m_fieldType = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
};

class TField : public TFieldType {
public:
	unsigned int m_data;
	TField() {
		m_data = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
};

class TCString {
public:
	std::string m_data;
	TCString() {
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
	bool BiggerThan(const TCString& s) const;
	int CountLF() const;
	int Length() const {
		return (int)m_data.length();
	}
};

class TCStringField : public TField {
public:
	TCString m_text;
	TCStringField() {
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
	bool BiggerThan(const TCStringField& s) const {
		return m_text.BiggerThan(s.m_text);
	}
};

class TClassEntry {
public:
	unsigned short m_constant;
	std::string m_className;
	TClassEntry() {
		m_constant = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
};

class TRepeatEvent : public TFieldType {
public:
	unsigned short m_dateExceptions;
	std::vector<unsigned int> m_exceptionEntry;
	unsigned short m_repeatEventFlag;
	TClassEntry m_classEntry;
	unsigned int m_brand;
	unsigned int m_interval;
	unsigned int m_endDate;
	unsigned int m_firstDayOfWeek;
	unsigned int m_dayIndex;
	unsigned char m_dayMask;
	unsigned int m_weekIndex;
	unsigned int m_dayNumber;
	unsigned int m_monthIndex;
	TRepeatEvent() {
		m_dateExceptions = 0;
		m_repeatEventFlag = 0;
		m_brand = 0;
		m_interval = 0;
		m_endDate = 0;
		m_firstDayOfWeek = 0;
		m_dayIndex = 0;
		m_dayMask = 0;
		m_weekIndex = 0;
		m_dayNumber = 0;
		m_monthIndex = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
	bool GetRRULE(std::string *outStr);
};

#define REPEAT_BRABD_DAILY				1
#define REPEAT_BRABD_WEEKLY				2
#define REPEAT_BRABD_MONTHLYBYDAY		3
#define REPEAT_BRABD_MONTHLYBYDATE		4
#define REPEAT_BRABD_YEARLYBYDATE		5
//#define REPEAT_BRABD_YEARLYBYDAY		6


class TDateBookEntry {
public:
	TField m_recordID;
	TField m_statusField;
	TField m_position;
	TField m_startTime;
	TField m_endTime;
	TCStringField m_description;
	TField m_duration;
	TCStringField m_note;
	TField m_untimed;
	TField m_private;
	TField m_category;
	TField m_alarmSet;
	TField m_alarmAdvUnits;
	TField m_alarmAdvType;
	TRepeatEvent m_repeatEvent;
	// programmer data
	void *m_refCon;
	TDateBookEntry() {
		m_refCon = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
	enum VEVENT_ENCODE {
		VEVENT_ENCODE_MIME_QP,
		VEVENT_ENCODE_BACKSLASH_ESCAPE
	};
	std::string GetSummary(VEVENT_ENCODE encode);
	bool GetVEVENT(std::vector<std::string>* lines, bool bUTC, VEVENT_ENCODE encode);
	std::string GetVEVENT_MakeEncodedLine(std::string data, std::string tag, VEVENT_ENCODE encode);
};

class TCategoryEntry {
public:
	unsigned int m_index;
	unsigned int m_id;
	unsigned int m_dirtyFlag;
	TCString m_longName;
	TCString m_shortName;
	TCategoryEntry() {
		m_index = 0;
		m_id = 0;
		m_dirtyFlag = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
};

class TDateBook {
public:
	unsigned int m_versionTag;
	TCString m_fileName;
	TCString m_tableString;
	unsigned int m_nextFree;
	unsigned int m_categoryCount;
	std::vector<TCategoryEntry> m_categoryEntry;
	unsigned int m_resourceID;
	unsigned int m_fieldsPerRow;
	unsigned int m_recIDPos;
	unsigned int m_recStatusPos;
	unsigned int m_placementPos;
	unsigned short m_fieldCount;
	std::vector<unsigned short> m_fieldEntry;
	unsigned int m_numEntries;
	std::vector<TDateBookEntry> m_dateBookEntry;
	TDateBook() {
		m_versionTag = 0;
		m_nextFree = 0;
		m_categoryCount = 0;
		m_resourceID = 0;
		m_fieldsPerRow = 0;
		m_recIDPos = 0;
		m_recStatusPos = 0;
		m_placementPos = 0;
		m_fieldCount = 0;
		m_numEntries = 0;
	}
	bool Read(std::istream *fs);
	bool Write(std::ostream *fs);
	bool Load(std::string path);
	bool Save(std::string path);
	int GetDateBookEntryCount();
	TDateBookEntry *GetNthDateBookEntry(int n);
};

class TUtil {
public:
	static std::string MIMEEncode(std::string inStr);
	static std::string BackslashEscapeEncode(std::string inStr);
};

}	// namespace
